package ink.wgink.service.user.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.exceptions.SearchException;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.user.UserDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.service.user.dao.IUserDao;
import ink.wgink.service.user.pojo.vos.UserVO;
import ink.wgink.service.user.service.IUserService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: UserServiceImpl
 * @Description: 用户
 * @Author: WangGeng
 * @Date: 2021/1/24 20:25
 * @Version: 1.0
 **/
@Service
public class UserServiceImpl extends DefaultBaseService implements IUserService {

    @Autowired
    private IUserDao userDao;
    @Autowired
    private PasswordEncoder passwordEncoder;

    @Override
    public synchronized void save(UserVO userVO) {
        saveUser(userVO, false);
    }

    @Override
    public void register(UserVO userVO) {
        saveUser(userVO, true);
    }

    /**
     * 保存用户
     *
     * @param userVO
     * @param isRegister 是否注册
     */
    private void saveUser(UserVO userVO, boolean isRegister) {
        UserDTO userDTO = get(userVO.getUserUsername());
        if (userDTO != null) {
            throw new SearchException("用户已经存在");
        }
        userVO.setUserPassword(passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex(userVO.getUserPassword())))));
        String userId = UUIDUtil.getUUID();
        Map<String, Object> params = HashMapUtil.beanToMap(userVO);
        params.put("userId", userId);
        if (isRegister) {
            setSaveInfoByUserId(params, userId);
        } else {
            setSaveInfo(params);
        }
        userDao.save(params);
    }

    @Override
    public void remove(List<String> ids) {
        Map<String, Object> params = getHashMap(2);
        params.put("ids", ids);
        setUpdateInfo(params);
        userDao.remove(params);
    }

    @Override
    public void update(String userId, UserVO userVO) {
        if (!StringUtils.isBlank(userVO.getUserPassword())) {
            userVO.setUserPassword(passwordEncoder.encode(DigestUtils.md5Hex(DigestUtils.md5Hex(DigestUtils.md5Hex(userVO.getUserPassword())))));
        }
        Map<String, Object> params = HashMapUtil.beanToMap(userVO);
        setUpdateInfo(params);
        userDao.update(params);
    }

    @Override
    public UserDTO get(String userId) {
        Map<String, Object> params = getHashMap(2);
        params.put("userId", userId);
        return userDao.get(params);
    }

    @Override
    public UserDTO getByUsername(String username) {
        Map<String, Object> params = getHashMap(2);
        params.put("userUsername", username);
        return userDao.get(params);
    }

    @Override
    public List<UserDTO> listByUserIds(List<String> userIds) {
        Map<String, Object> params = getHashMap(2);
        params.put("userIds", userIds);
        return list(params);
    }

    @Override
    public List<UserDTO> list(Map<String, Object> params) {
        return userDao.list(params);
    }

    @Override
    public SuccessResultList<List<UserDTO>> listPage(ListPage page) {
        PageHelper.startPage(page.getPage(), page.getRows());
        List<UserDTO> userDTOs = list(page.getParams());
        PageInfo<UserDTO> pageInfo = new PageInfo<>(userDTOs);
        return new SuccessResultList<>(userDTOs, pageInfo.getPageNum(), pageInfo.getTotal());
    }

}
